/**
 * Windows API header module
 *
 * Translated from MinGW API for MS-Windows 3.10
 *
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost License 1.0)
 * Source: $(DRUNTIMESRC src/core/sys/windows/_ole2ver.d)
 */
module core.sys.windows.ole2ver;
version (Windows):

// These are apparently not documented on the MSDN site
enum rmm = 23;
enum rup = 639;
