module std.internal.unicode_comp;
import std.internal.unicode_tables;

@safe pure nothrow @nogc package(std):

static if (size_t.sizeof == 8)
{
    //6976 bytes
    enum combiningClassTrieEntries = TrieEntry!(ubyte, 8, 7, 6)([0x0, 0x20,
            0x120], [0x100, 0x400, 0x1240], [0x402030202020100,
            0x206020202020205, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x202020202020202, 0x202020202020202,
            0x202020202020202, 0x0, 0x0, 0x0, 0x20001, 0x300000000,
            0x5000400000000, 0x8000000070006, 0xb0000000a0009, 0xe0000000d000c,
            0x11000f0010000f, 0x11000f0011000f, 0x1100000011000f,
            0x11000f00120000, 0x13000000110000, 0x17001600150014,
            0x1b001a00190018, 0x1d0000001c, 0x0, 0x0, 0x1e0000, 0x0, 0x0, 0x0,
            0x2000000000001f, 0x2100000000, 0x22, 0x240023, 0x28002700260025,
            0x2a000000000029, 0x2b000000000000, 0x0, 0x0, 0x2c000000000000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x2d000000000000, 0x2f0000002e0000, 0x0, 0x0, 0x3100000030, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x34003300320000, 0x0, 0x36000000000035, 0x3a003900380037,
            0x3c003b00000000, 0x3d000000000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x3e, 0x0, 0x0, 0x3f, 0x0, 0x0, 0x40000000000000, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x4200350000, 0x3a000000000043, 0x0, 0x0, 0x0, 0x0, 0x4400000000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x4600450000, 0x470000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e6e6e6e6e6e6,
            0xe6e6e6e6e6e6e6e6, 0xdcdce8e6e6e6e6e6, 0xdcdcdcdcd8e8dcdc,
            0xcadcdcdcdccacadc, 0xdcdcdcdcdcdcdcca, 0x1010101dcdcdcdc,
            0xe6e6e6dcdcdcdc01, 0xdce6f0e6e6e6e6e6, 0xdcdce6e6e6dcdc,
            0xe6dcdcdcdce6e6e6, 0xe9eaeae9e6dcdce8, 0xe6e6e6e6e6e9eaea,
            0xe6e6e6e6e6e6e6e6, 0x0, 0x0, 0xe6e6e6e6e6000000, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6dce6e6e6e6dc00,
            0xe6e6e6e6dcdee6e6, 0xdcdcdcdcdcdce6e6, 0xe6e4dee6e6dce6e6,
            0x11100f0e0d0c0b0a, 0x1700161514131312, 0x1200dce600191800, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e6e6e6e6e6e6,
            0x201f1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f1e1d1c1b000000,
            0xe6dcdce6e6222120, 0xdce6e6dce6e6e6e6, 0x0, 0x0, 0x23, 0x0, 0x0,
            0x0, 0xe6e6000000000000, 0xe60000e6e6e6e6e6, 0xe60000e6dce6e6e6,
            0xdce6e6dc00e6, 0x0, 0x0, 0x0, 0x0, 0x2400, 0x0, 0x0, 0x0,
            0xdce6e6dce6e6dce6, 0xe6dce6dcdce6dcdc, 0xe6dce6dce6dce6e6,
            0xe6e6dc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xe6e6e6e6e6000000, 0xe6dce6e6, 0x0, 0x0, 0x0, 0xe6e6000000000000,
            0xe6e6e6e6e600e6e6, 0xe6e6e600e6e6e6e6, 0xe6e6e6e6e600, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xdcdcdc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xe6dce6e600000000, 0xdcdcdce6e6e6dce6, 0xe6dce6e6e61d1c1b,
            0xe6e6e6e6dcdce6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x700000000,
            0x0, 0x90000000000, 0xe6e6dce600, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x90000000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x90000000000,
            0x5b540000000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x90000, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x96767,
            0x0, 0x6b6b6b6b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x7676, 0x0, 0x7a7a7a7a, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0xdcdc, 0x0, 0x0, 0xdc00dc0000000000, 0xd800,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8400828100, 0x828282820000,
            0xe6e60009e6e60082, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xdc000000000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x700000000000000, 0x90900, 0x0, 0xdc0000000000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e60000000000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x900000000, 0x0, 0x0, 0x0,
            0x900000000, 0x0, 0x0, 0x0, 0x90000, 0xe60000000000, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xdce6de00, 0x0, 0x0, 0xe600000000000000, 0xdc, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9, 0x0, 0xe6e6e60000000000,
            0xdc0000e6e6e6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x700000000, 0x0,
            0x900000000, 0x0, 0x0, 0x0, 0x0, 0xe6e6e6dce6000000, 0xe6e6e6e6,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9090000, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x7000000000000, 0x0, 0x9090000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x700000000000000, 0x0, 0x0, 0x0, 0xdcdcdc0100e6e6e6,
            0xdcdcdcdce6e6dcdc, 0x1010101010100e6, 0xdc0000000001,
            0xe600000000, 0x0, 0xe6e6e6e6e6dce6e6, 0xdcd6eae6e6dce6e6,
            0xe6e6e6e6e6e6e6ca, 0xe6e6e6e6e6e6e6e6, 0xe6e6e6e6e6e6e6, 0x0, 0x0,
            0xdce6dce900000000, 0x0, 0x0, 0xe6e6e6e60101e6e6, 0xe6e6010101,
            0xe60101000000e600, 0xdcdcdcdc0101e6dc, 0xe6, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0xe600000000000000, 0xe6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x900000000000000, 0x0, 0x0, 0x0, 0x0,
            0xe6e6e6e6e6e6e6e6, 0xe6e6e6e6e6e6e6e6, 0xe6e6e6e6e6e6e6e6,
            0xe6e6e6e6e6e6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe0e0dee8e4da0000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x80800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xe600000000000000, 0xe6e6e6e600000000,
            0xe6e6e6e6e6e6, 0x0, 0x0, 0x0, 0xe600000000000000, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6, 0x0, 0x9000000000000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x900000000, 0x0, 0x0, 0x0,
            0xe6e6e6e6e6e6e6e6, 0xe6e6e6e6e6e6e6e6, 0xe6e6, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0xdcdcdc000000, 0x0, 0x0, 0x0, 0x0, 0x9000000, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7000000, 0x0, 0x9,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xe60000dce6e600e6, 0xe6e60000000000e6, 0xe600, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x9000000000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x90000000000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x1a000000000000, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xe6e6e6e6e6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xdc0000000000, 0x0, 0xe600dc0000000000, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x900000000dc01e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x70900, 0xe6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x909000000, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x709000000000000, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x1d8d80000000000, 0xd8d8e20000000101, 0xd8d8d8,
            0xdcdcdcdcdc000000, 0xe6e6e60000dcdcdc, 0xdcdce6e6, 0x0, 0x0, 0x0,
            0xe6e6e6e60000, 0x0, 0x0, 0xe6e6e60000, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]);
    enum composeIdxMask = (1 << 11) - 1, composeCntShift = 11;
    enum compositionJumpTrieEntries = TrieEntry!(ushort, 12, 9)([0x0, 0x400],
            [0x1000, 0x2000], [0x3000200010000, 0x7000600050004,
            0x7000700070008, 0xa000700090007, 0x70007000c000b, 0x7000700070007,
            0x700070007000d, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x700070007000e, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0x7000700070007, 0x7000700070007,
            0x7000700070007, 0x7000700070007, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff080208010800,
            0x281618138003ffff, 0x383308328821301b, 0x285108507841383a,
            0x8068485f185c3056, 0x3882407affff1078, 0x30a510a398903889,
            0xffff30b648ad10ab, 0xffffffffffffffff, 0x28cf18cc80bcffff,
            0x38ec08eb88da30d4, 0x290b110970fb40f3, 0x8122491919163110,
            0x393c4134ffff1132, 0x3960115e994b4143, 0xffff317351691167,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffff1979,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff217cffffffff,
            0x984118209810980, 0xffff2185ffffffff, 0x989ffffffffffff,
            0xffffffffffffffff, 0xffff0991198e218a, 0xffffffffffff0992,
            0xffffffffffff2193, 0xffff2197ffffffff, 0x99f119d099c099b,
            0xffff21a0ffffffff, 0x9a4ffffffffffff, 0xffffffffffffffff,
            0xffff09ac19a921a5, 0xffffffffffff09ad, 0xffffffffffff21ae,
            0x21b621b2ffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0x11bc11baffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffff11c011be, 0xffffffffffffffff,
            0xffffffffffffffff, 0x9c309c2ffffffff, 0xffffffffffffffff,
            0xffffffff09c509c4, 0xffffffffffffffff, 0x9c909c809c709c6,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0x9caffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffff29d029cb, 0xffffffffffffffff,
            0xffffffffffffffff, 0x29d5ffffffffffff, 0xffffffffffff29da,
            0x9dfffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0x9e109e0ffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0x9e309e2ffffffff, 0xffffffff09e509e4,
            0x9e709e6ffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffff09e8ffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff39e9ffff,
            0x29f4ffff21f0ffff, 0xffffffff39f9ffff, 0x2200ffffffffffff,
            0xffffffff0a04ffff, 0xffffffff3205ffff, 0xffffffff2a0bffff,
            0xffff0a11ffff0a10, 0xffffffff4212ffff, 0x321effff221affff,
            0xffffffff4224ffff, 0x222cffffffffffff, 0xffffffff1230ffff,
            0xffffffff4232ffff, 0x1a431a40323affff, 0xffff0a46ffffffff,
            0xffff1247ffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffff0a49ffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xa4cffffffff124a, 0xa5212501a4dffff,
            0xffff0a57ffff2253, 0xffff0a58ffffffff, 0x2259ffffffffffff,
            0xa5dffffffffffff, 0xa5effffffffffff, 0xffffffff0a5fffff,
            0xa62ffffffff1260, 0xa6812661a63ffff, 0xffff0a6dffff2269,
            0xffff0a6effffffff, 0x226fffffffffffff, 0xa73ffffffffffff,
            0xa74ffffffffffff, 0xffffffff0a75ffff, 0xffffffffffffffff,
            0xffff0a76ffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff0a780a77,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffff0a7a0a79, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffff0a7c0a7b, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0x1a7dffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff0a81ffff0a80,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff0a82ffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffff0a83ffffffff, 0xffffffff0a84ffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffff0a85, 0xffffffffffffffff, 0xa87ffffffff0a86,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0x1288ffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0x1a8affffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffff0a8dffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xa90128effffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffff0a91ffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xa92ffffffffffff, 0xffffffffffffffff,
            0xffff1a93ffffffff, 0xffff0a96ffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xa991297ffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffff1a9affff, 0xffffffffffff0a9d, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffff0a9effff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xaa0ffff0a9fffff, 0xaa2ffff0aa1ffff,
            0xffffffff0aa3ffff, 0xffffffff0aa4ffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff0aa5ffffffff,
            0xaa80aa7ffff0aa6, 0xffff0aa9ffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xaab0aaaffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xaad0aacffffffff,
            0xffffffffffffffff, 0xaaf0aaeffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff12b212b0,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff0ab50ab4,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffff0ab70ab6, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xac10ac022bc22b8,
            0xac50ac40ac30ac2, 0xacf0ace22ca22c6, 0xad30ad20ad10ad0,
            0xffffffff12d612d4, 0xffffffffffffffff, 0xffffffff12da12d8,
            0xffffffffffffffff, 0xae50ae422e022dc, 0xae90ae80ae70ae6,
            0xaf30af222ee22ea, 0xaf70af60af50af4, 0xffffffff1afb1af8,
            0xffffffffffffffff, 0xffffffff1b011afe, 0xffffffffffffffff,
            0xffffffff13061304, 0xffffffffffffffff, 0xffffffff130a1308,
            0xffffffffffffffff, 0xffffffff1b0f1b0c, 0xffffffffffffffff,
            0xffffffff1b12ffff, 0xffffffffffffffff, 0xb1e0b1d23192315,
            0xb220b210b200b1f, 0xb2c0b2b23272323, 0xb300b2f0b2e0b2d,
            0xffffffffffff0b31, 0xffffffffffff0b32, 0xffffffffffffffff,
            0xffffffffffff0b33, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff0b34ffffffff,
            0xffffffffffffffff, 0x1b35ffffffffffff, 0xffffffffffffffff,
            0xffff0b38ffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffff0b39ffffffff, 0xffffffffffffffff, 0xffff1b3affffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffff0b3effff0b3d, 0xffffffffffff0b3f,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff0b41ffff0b40,
            0xffffffffffff0b42, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xb43ffffffffffff,
            0xffffffffffffffff, 0xb45ffffffff0b44, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xb46ffffffffffff, 0xffffffff0b47ffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffff0b48,
            0xb49ffffffffffff, 0xffffffff0b4affff, 0xffffffffffff0b4b,
            0xffffffff0b4cffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff0b4dffff,
            0xffffffff0b4f0b4e, 0xffffffffffffffff, 0xffffffffffffffff,
            0xb510b50ffffffff, 0xb530b52ffffffff, 0xb550b54ffffffff,
            0xffffffff0b570b56, 0xb590b58ffffffff, 0xb5b0b5affffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffff0b5d0b5cffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffff0b5effffffff, 0xffffffffffffffff, 0xb61ffff0b600b5f,
            0xffffffffffffffff, 0xb630b62ffffffff, 0xffffffff0b650b64,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffff0b66ffffffff, 0xb67ffffffffffff, 0xb69ffff0b68ffff,
            0xb6bffff0b6affff, 0xb6dffff0b6cffff, 0xb6fffff0b6effff,
            0xb71ffff0b70ffff, 0xffffffff0b72ffff, 0xffff0b74ffff0b73,
            0xffffffffffff0b75, 0x1376ffffffffffff, 0xffff1378ffffffff,
            0xffffffff137affff, 0x137effffffff137c, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffff0b80ffff, 0xffffffffffffffff,
            0xffff0b81ffffffff, 0xb82ffffffffffff, 0xb84ffff0b83ffff,
            0xb86ffff0b85ffff, 0xb88ffff0b87ffff, 0xb8affff0b89ffff,
            0xb8cffff0b8bffff, 0xffffffff0b8dffff, 0xffff0b8fffff0b8e,
            0xffffffffffff0b90, 0x1391ffffffffffff, 0xffff1393ffffffff,
            0xffffffff1395ffff, 0x1399ffffffff1397, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xb9bffffffffffff, 0xffff0b9e0b9d0b9c, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffff0b9fffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xba1ffff0ba0ffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffff0ba2ffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffff0ba40ba3ffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff,
            0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff, 0xffffffffffffffff]);
    @property immutable(CompEntry[]) compositionTable()
    {
        alias CE = CompEntry;
        static immutable CE[] t = [
            CE(0x00338, 0x0226e), CE(0x00338, 0x02260), CE(0x00338, 0x0226f),
            CE(0x00300, 0x000c0), CE(0x00301, 0x000c1), CE(0x00302, 0x000c2),
            CE(0x00303, 0x000c3), CE(0x00304, 0x00100), CE(0x00306, 0x00102),
            CE(0x00307, 0x00226), CE(0x00308, 0x000c4), CE(0x00309, 0x01ea2),
            CE(0x0030a, 0x000c5), CE(0x0030c, 0x001cd), CE(0x0030f, 0x00200),
            CE(0x00311, 0x00202), CE(0x00323, 0x01ea0), CE(0x00325, 0x01e00),
            CE(0x00328, 0x00104), CE(0x00307, 0x01e02), CE(0x00323, 0x01e04),
            CE(0x00331, 0x01e06), CE(0x00301, 0x00106), CE(0x00302, 0x00108),
            CE(0x00307, 0x0010a), CE(0x0030c, 0x0010c), CE(0x00327, 0x000c7),
            CE(0x00307, 0x01e0a), CE(0x0030c, 0x0010e), CE(0x00323, 0x01e0c),
            CE(0x00327, 0x01e10), CE(0x0032d, 0x01e12), CE(0x00331, 0x01e0e),
            CE(0x00300, 0x000c8), CE(0x00301, 0x000c9), CE(0x00302, 0x000ca),
            CE(0x00303, 0x01ebc), CE(0x00304, 0x00112), CE(0x00306, 0x00114),
            CE(0x00307, 0x00116), CE(0x00308, 0x000cb), CE(0x00309, 0x01eba),
            CE(0x0030c, 0x0011a), CE(0x0030f, 0x00204), CE(0x00311, 0x00206),
            CE(0x00323, 0x01eb8), CE(0x00327, 0x00228), CE(0x00328, 0x00118),
            CE(0x0032d, 0x01e18), CE(0x00330, 0x01e1a), CE(0x00307, 0x01e1e),
            CE(0x00301, 0x001f4), CE(0x00302, 0x0011c), CE(0x00304, 0x01e20),
            CE(0x00306, 0x0011e), CE(0x00307, 0x00120), CE(0x0030c, 0x001e6),
            CE(0x00327, 0x00122), CE(0x00302, 0x00124), CE(0x00307, 0x01e22),
            CE(0x00308, 0x01e26), CE(0x0030c, 0x0021e), CE(0x00323, 0x01e24),
            CE(0x00327, 0x01e28), CE(0x0032e, 0x01e2a), CE(0x00300, 0x000cc),
            CE(0x00301, 0x000cd), CE(0x00302, 0x000ce), CE(0x00303, 0x00128),
            CE(0x00304, 0x0012a), CE(0x00306, 0x0012c), CE(0x00307, 0x00130),
            CE(0x00308, 0x000cf), CE(0x00309, 0x01ec8), CE(0x0030c, 0x001cf),
            CE(0x0030f, 0x00208), CE(0x00311, 0x0020a), CE(0x00323, 0x01eca),
            CE(0x00328, 0x0012e), CE(0x00330, 0x01e2c), CE(0x00302, 0x00134),
            CE(0x00301, 0x01e30), CE(0x0030c, 0x001e8), CE(0x00323, 0x01e32),
            CE(0x00327, 0x00136), CE(0x00331, 0x01e34), CE(0x00301, 0x00139),
            CE(0x0030c, 0x0013d), CE(0x00323, 0x01e36), CE(0x00327, 0x0013b),
            CE(0x0032d, 0x01e3c), CE(0x00331, 0x01e3a), CE(0x00301, 0x01e3e),
            CE(0x00307, 0x01e40), CE(0x00323, 0x01e42), CE(0x00300, 0x001f8),
            CE(0x00301, 0x00143), CE(0x00303, 0x000d1), CE(0x00307, 0x01e44),
            CE(0x0030c, 0x00147), CE(0x00323, 0x01e46), CE(0x00327, 0x00145),
            CE(0x0032d, 0x01e4a), CE(0x00331, 0x01e48), CE(0x00300, 0x000d2),
            CE(0x00301, 0x000d3), CE(0x00302, 0x000d4), CE(0x00303, 0x000d5),
            CE(0x00304, 0x0014c), CE(0x00306, 0x0014e), CE(0x00307, 0x0022e),
            CE(0x00308, 0x000d6), CE(0x00309, 0x01ece), CE(0x0030b, 0x00150),
            CE(0x0030c, 0x001d1), CE(0x0030f, 0x0020c), CE(0x00311, 0x0020e),
            CE(0x0031b, 0x001a0), CE(0x00323, 0x01ecc), CE(0x00328, 0x001ea),
            CE(0x00301, 0x01e54), CE(0x00307, 0x01e56), CE(0x00301, 0x00154),
            CE(0x00307, 0x01e58), CE(0x0030c, 0x00158), CE(0x0030f, 0x00210),
            CE(0x00311, 0x00212), CE(0x00323, 0x01e5a), CE(0x00327, 0x00156),
            CE(0x00331, 0x01e5e), CE(0x00301, 0x0015a), CE(0x00302, 0x0015c),
            CE(0x00307, 0x01e60), CE(0x0030c, 0x00160), CE(0x00323, 0x01e62),
            CE(0x00326, 0x00218), CE(0x00327, 0x0015e), CE(0x00307, 0x01e6a),
            CE(0x0030c, 0x00164), CE(0x00323, 0x01e6c), CE(0x00326, 0x0021a),
            CE(0x00327, 0x00162), CE(0x0032d, 0x01e70), CE(0x00331, 0x01e6e),
            CE(0x00300, 0x000d9), CE(0x00301, 0x000da), CE(0x00302, 0x000db),
            CE(0x00303, 0x00168), CE(0x00304, 0x0016a), CE(0x00306, 0x0016c),
            CE(0x00308, 0x000dc), CE(0x00309, 0x01ee6), CE(0x0030a, 0x0016e),
            CE(0x0030b, 0x00170), CE(0x0030c, 0x001d3), CE(0x0030f, 0x00214),
            CE(0x00311, 0x00216), CE(0x0031b, 0x001af), CE(0x00323, 0x01ee4),
            CE(0x00324, 0x01e72), CE(0x00328, 0x00172), CE(0x0032d, 0x01e76),
            CE(0x00330, 0x01e74), CE(0x00303, 0x01e7c), CE(0x00323, 0x01e7e),
            CE(0x00300, 0x01e80), CE(0x00301, 0x01e82), CE(0x00302, 0x00174),
            CE(0x00307, 0x01e86), CE(0x00308, 0x01e84), CE(0x00323, 0x01e88),
            CE(0x00307, 0x01e8a), CE(0x00308, 0x01e8c), CE(0x00300, 0x01ef2),
            CE(0x00301, 0x000dd), CE(0x00302, 0x00176), CE(0x00303, 0x01ef8),
            CE(0x00304, 0x00232), CE(0x00307, 0x01e8e), CE(0x00308, 0x00178),
            CE(0x00309, 0x01ef6), CE(0x00323, 0x01ef4), CE(0x00301, 0x00179),
            CE(0x00302, 0x01e90), CE(0x00307, 0x0017b), CE(0x0030c, 0x0017d),
            CE(0x00323, 0x01e92), CE(0x00331, 0x01e94), CE(0x00300, 0x000e0),
            CE(0x00301, 0x000e1), CE(0x00302, 0x000e2), CE(0x00303, 0x000e3),
            CE(0x00304, 0x00101), CE(0x00306, 0x00103), CE(0x00307, 0x00227),
            CE(0x00308, 0x000e4), CE(0x00309, 0x01ea3), CE(0x0030a, 0x000e5),
            CE(0x0030c, 0x001ce), CE(0x0030f, 0x00201), CE(0x00311, 0x00203),
            CE(0x00323, 0x01ea1), CE(0x00325, 0x01e01), CE(0x00328, 0x00105),
            CE(0x00307, 0x01e03), CE(0x00323, 0x01e05), CE(0x00331, 0x01e07),
            CE(0x00301, 0x00107), CE(0x00302, 0x00109), CE(0x00307, 0x0010b),
            CE(0x0030c, 0x0010d), CE(0x00327, 0x000e7), CE(0x00307, 0x01e0b),
            CE(0x0030c, 0x0010f), CE(0x00323, 0x01e0d), CE(0x00327, 0x01e11),
            CE(0x0032d, 0x01e13), CE(0x00331, 0x01e0f), CE(0x00300, 0x000e8),
            CE(0x00301, 0x000e9), CE(0x00302, 0x000ea), CE(0x00303, 0x01ebd),
            CE(0x00304, 0x00113), CE(0x00306, 0x00115), CE(0x00307, 0x00117),
            CE(0x00308, 0x000eb), CE(0x00309, 0x01ebb), CE(0x0030c, 0x0011b),
            CE(0x0030f, 0x00205), CE(0x00311, 0x00207), CE(0x00323, 0x01eb9),
            CE(0x00327, 0x00229), CE(0x00328, 0x00119), CE(0x0032d, 0x01e19),
            CE(0x00330, 0x01e1b), CE(0x00307, 0x01e1f), CE(0x00301, 0x001f5),
            CE(0x00302, 0x0011d), CE(0x00304, 0x01e21), CE(0x00306, 0x0011f),
            CE(0x00307, 0x00121), CE(0x0030c, 0x001e7), CE(0x00327, 0x00123),
            CE(0x00302, 0x00125), CE(0x00307, 0x01e23), CE(0x00308, 0x01e27),
            CE(0x0030c, 0x0021f), CE(0x00323, 0x01e25), CE(0x00327, 0x01e29),
            CE(0x0032e, 0x01e2b), CE(0x00331, 0x01e96), CE(0x00300, 0x000ec),
            CE(0x00301, 0x000ed), CE(0x00302, 0x000ee), CE(0x00303, 0x00129),
            CE(0x00304, 0x0012b), CE(0x00306, 0x0012d), CE(0x00308, 0x000ef),
            CE(0x00309, 0x01ec9), CE(0x0030c, 0x001d0), CE(0x0030f, 0x00209),
            CE(0x00311, 0x0020b), CE(0x00323, 0x01ecb), CE(0x00328, 0x0012f),
            CE(0x00330, 0x01e2d), CE(0x00302, 0x00135), CE(0x0030c, 0x001f0),
            CE(0x00301, 0x01e31), CE(0x0030c, 0x001e9), CE(0x00323, 0x01e33),
            CE(0x00327, 0x00137), CE(0x00331, 0x01e35), CE(0x00301, 0x0013a),
            CE(0x0030c, 0x0013e), CE(0x00323, 0x01e37), CE(0x00327, 0x0013c),
            CE(0x0032d, 0x01e3d), CE(0x00331, 0x01e3b), CE(0x00301, 0x01e3f),
            CE(0x00307, 0x01e41), CE(0x00323, 0x01e43), CE(0x00300, 0x001f9),
            CE(0x00301, 0x00144), CE(0x00303, 0x000f1), CE(0x00307, 0x01e45),
            CE(0x0030c, 0x00148), CE(0x00323, 0x01e47), CE(0x00327, 0x00146),
            CE(0x0032d, 0x01e4b), CE(0x00331, 0x01e49), CE(0x00300, 0x000f2),
            CE(0x00301, 0x000f3), CE(0x00302, 0x000f4), CE(0x00303, 0x000f5),
            CE(0x00304, 0x0014d), CE(0x00306, 0x0014f), CE(0x00307, 0x0022f),
            CE(0x00308, 0x000f6), CE(0x00309, 0x01ecf), CE(0x0030b, 0x00151),
            CE(0x0030c, 0x001d2), CE(0x0030f, 0x0020d), CE(0x00311, 0x0020f),
            CE(0x0031b, 0x001a1), CE(0x00323, 0x01ecd), CE(0x00328, 0x001eb),
            CE(0x00301, 0x01e55), CE(0x00307, 0x01e57), CE(0x00301, 0x00155),
            CE(0x00307, 0x01e59), CE(0x0030c, 0x00159), CE(0x0030f, 0x00211),
            CE(0x00311, 0x00213), CE(0x00323, 0x01e5b), CE(0x00327, 0x00157),
            CE(0x00331, 0x01e5f), CE(0x00301, 0x0015b), CE(0x00302, 0x0015d),
            CE(0x00307, 0x01e61), CE(0x0030c, 0x00161), CE(0x00323, 0x01e63),
            CE(0x00326, 0x00219), CE(0x00327, 0x0015f), CE(0x00307, 0x01e6b),
            CE(0x00308, 0x01e97), CE(0x0030c, 0x00165), CE(0x00323, 0x01e6d),
            CE(0x00326, 0x0021b), CE(0x00327, 0x00163), CE(0x0032d, 0x01e71),
            CE(0x00331, 0x01e6f), CE(0x00300, 0x000f9), CE(0x00301, 0x000fa),
            CE(0x00302, 0x000fb), CE(0x00303, 0x00169), CE(0x00304, 0x0016b),
            CE(0x00306, 0x0016d), CE(0x00308, 0x000fc), CE(0x00309, 0x01ee7),
            CE(0x0030a, 0x0016f), CE(0x0030b, 0x00171), CE(0x0030c, 0x001d4),
            CE(0x0030f, 0x00215), CE(0x00311, 0x00217), CE(0x0031b, 0x001b0),
            CE(0x00323, 0x01ee5), CE(0x00324, 0x01e73), CE(0x00328, 0x00173),
            CE(0x0032d, 0x01e77), CE(0x00330, 0x01e75), CE(0x00303, 0x01e7d),
            CE(0x00323, 0x01e7f), CE(0x00300, 0x01e81), CE(0x00301, 0x01e83),
            CE(0x00302, 0x00175), CE(0x00307, 0x01e87), CE(0x00308, 0x01e85),
            CE(0x0030a, 0x01e98), CE(0x00323, 0x01e89), CE(0x00307, 0x01e8b),
            CE(0x00308, 0x01e8d), CE(0x00300, 0x01ef3), CE(0x00301, 0x000fd),
            CE(0x00302, 0x00177), CE(0x00303, 0x01ef9), CE(0x00304, 0x00233),
            CE(0x00307, 0x01e8f), CE(0x00308, 0x000ff), CE(0x00309, 0x01ef7),
            CE(0x0030a, 0x01e99), CE(0x00323, 0x01ef5), CE(0x00301, 0x0017a),
            CE(0x00302, 0x01e91), CE(0x00307, 0x0017c), CE(0x0030c, 0x0017e),
            CE(0x00323, 0x01e93), CE(0x00331, 0x01e95), CE(0x00300, 0x01fed),
            CE(0x00301, 0x00385), CE(0x00342, 0x01fc1), CE(0x00300, 0x01ea6),
            CE(0x00301, 0x01ea4), CE(0x00303, 0x01eaa), CE(0x00309, 0x01ea8),
            CE(0x00304, 0x001de), CE(0x00301, 0x001fa), CE(0x00301, 0x001fc),
            CE(0x00304, 0x001e2), CE(0x00301, 0x01e08), CE(0x00300, 0x01ec0),
            CE(0x00301, 0x01ebe), CE(0x00303, 0x01ec4), CE(0x00309, 0x01ec2),
            CE(0x00301, 0x01e2e), CE(0x00300, 0x01ed2), CE(0x00301, 0x01ed0),
            CE(0x00303, 0x01ed6), CE(0x00309, 0x01ed4), CE(0x00301, 0x01e4c),
            CE(0x00304, 0x0022c), CE(0x00308, 0x01e4e), CE(0x00304, 0x0022a),
            CE(0x00301, 0x001fe), CE(0x00300, 0x001db), CE(0x00301, 0x001d7),
            CE(0x00304, 0x001d5), CE(0x0030c, 0x001d9), CE(0x00300, 0x01ea7),
            CE(0x00301, 0x01ea5), CE(0x00303, 0x01eab), CE(0x00309, 0x01ea9),
            CE(0x00304, 0x001df), CE(0x00301, 0x001fb), CE(0x00301, 0x001fd),
            CE(0x00304, 0x001e3), CE(0x00301, 0x01e09), CE(0x00300, 0x01ec1),
            CE(0x00301, 0x01ebf), CE(0x00303, 0x01ec5), CE(0x00309, 0x01ec3),
            CE(0x00301, 0x01e2f), CE(0x00300, 0x01ed3), CE(0x00301, 0x01ed1),
            CE(0x00303, 0x01ed7), CE(0x00309, 0x01ed5), CE(0x00301, 0x01e4d),
            CE(0x00304, 0x0022d), CE(0x00308, 0x01e4f), CE(0x00304, 0x0022b),
            CE(0x00301, 0x001ff), CE(0x00300, 0x001dc), CE(0x00301, 0x001d8),
            CE(0x00304, 0x001d6), CE(0x0030c, 0x001da), CE(0x00300, 0x01eb0),
            CE(0x00301, 0x01eae), CE(0x00303, 0x01eb4), CE(0x00309, 0x01eb2),
            CE(0x00300, 0x01eb1), CE(0x00301, 0x01eaf), CE(0x00303, 0x01eb5),
            CE(0x00309, 0x01eb3), CE(0x00300, 0x01e14), CE(0x00301, 0x01e16),
            CE(0x00300, 0x01e15), CE(0x00301, 0x01e17), CE(0x00300, 0x01e50),
            CE(0x00301, 0x01e52), CE(0x00300, 0x01e51), CE(0x00301, 0x01e53),
            CE(0x00307, 0x01e64), CE(0x00307, 0x01e65), CE(0x00307, 0x01e66),
            CE(0x00307, 0x01e67), CE(0x00301, 0x01e78), CE(0x00301, 0x01e79),
            CE(0x00308, 0x01e7a), CE(0x00308, 0x01e7b), CE(0x00307, 0x01e9b),
            CE(0x00300, 0x01edc), CE(0x00301, 0x01eda), CE(0x00303, 0x01ee0),
            CE(0x00309, 0x01ede), CE(0x00323, 0x01ee2), CE(0x00300, 0x01edd),
            CE(0x00301, 0x01edb), CE(0x00303, 0x01ee1), CE(0x00309, 0x01edf),
            CE(0x00323, 0x01ee3), CE(0x00300, 0x01eea), CE(0x00301, 0x01ee8),
            CE(0x00303, 0x01eee), CE(0x00309, 0x01eec), CE(0x00323, 0x01ef0),
            CE(0x00300, 0x01eeb), CE(0x00301, 0x01ee9), CE(0x00303, 0x01eef),
            CE(0x00309, 0x01eed), CE(0x00323, 0x01ef1), CE(0x0030c, 0x001ee),
            CE(0x00304, 0x001ec), CE(0x00304, 0x001ed), CE(0x00304, 0x001e0),
            CE(0x00304, 0x001e1), CE(0x00306, 0x01e1c), CE(0x00306, 0x01e1d),
            CE(0x00304, 0x00230), CE(0x00304, 0x00231), CE(0x0030c, 0x001ef),
            CE(0x00300, 0x01fba), CE(0x00301, 0x00386), CE(0x00304, 0x01fb9),
            CE(0x00306, 0x01fb8), CE(0x00313, 0x01f08), CE(0x00314, 0x01f09),
            CE(0x00345, 0x01fbc), CE(0x00300, 0x01fc8), CE(0x00301, 0x00388),
            CE(0x00313, 0x01f18), CE(0x00314, 0x01f19), CE(0x00300, 0x01fca),
            CE(0x00301, 0x00389), CE(0x00313, 0x01f28), CE(0x00314, 0x01f29),
            CE(0x00345, 0x01fcc), CE(0x00300, 0x01fda), CE(0x00301, 0x0038a),
            CE(0x00304, 0x01fd9), CE(0x00306, 0x01fd8), CE(0x00308, 0x003aa),
            CE(0x00313, 0x01f38), CE(0x00314, 0x01f39), CE(0x00300, 0x01ff8),
            CE(0x00301, 0x0038c), CE(0x00313, 0x01f48), CE(0x00314, 0x01f49),
            CE(0x00314, 0x01fec), CE(0x00300, 0x01fea), CE(0x00301, 0x0038e),
            CE(0x00304, 0x01fe9), CE(0x00306, 0x01fe8), CE(0x00308, 0x003ab),
            CE(0x00314, 0x01f59), CE(0x00300, 0x01ffa), CE(0x00301, 0x0038f),
            CE(0x00313, 0x01f68), CE(0x00314, 0x01f69), CE(0x00345, 0x01ffc),
            CE(0x00345, 0x01fb4), CE(0x00345, 0x01fc4), CE(0x00300, 0x01f70),
            CE(0x00301, 0x003ac), CE(0x00304, 0x01fb1), CE(0x00306, 0x01fb0),
            CE(0x00313, 0x01f00), CE(0x00314, 0x01f01), CE(0x00342, 0x01fb6),
            CE(0x00345, 0x01fb3), CE(0x00300, 0x01f72), CE(0x00301, 0x003ad),
            CE(0x00313, 0x01f10), CE(0x00314, 0x01f11), CE(0x00300, 0x01f74),
            CE(0x00301, 0x003ae), CE(0x00313, 0x01f20), CE(0x00314, 0x01f21),
            CE(0x00342, 0x01fc6), CE(0x00345, 0x01fc3), CE(0x00300, 0x01f76),
            CE(0x00301, 0x003af), CE(0x00304, 0x01fd1), CE(0x00306, 0x01fd0),
            CE(0x00308, 0x003ca), CE(0x00313, 0x01f30), CE(0x00314, 0x01f31),
            CE(0x00342, 0x01fd6), CE(0x00300, 0x01f78), CE(0x00301, 0x003cc),
            CE(0x00313, 0x01f40), CE(0x00314, 0x01f41), CE(0x00313, 0x01fe4),
            CE(0x00314, 0x01fe5), CE(0x00300, 0x01f7a), CE(0x00301, 0x003cd),
            CE(0x00304, 0x01fe1), CE(0x00306, 0x01fe0), CE(0x00308, 0x003cb),
            CE(0x00313, 0x01f50), CE(0x00314, 0x01f51), CE(0x00342, 0x01fe6),
            CE(0x00300, 0x01f7c), CE(0x00301, 0x003ce), CE(0x00313, 0x01f60),
            CE(0x00314, 0x01f61), CE(0x00342, 0x01ff6), CE(0x00345, 0x01ff3),
            CE(0x00300, 0x01fd2), CE(0x00301, 0x00390), CE(0x00342, 0x01fd7),
            CE(0x00300, 0x01fe2), CE(0x00301, 0x003b0), CE(0x00342, 0x01fe7),
            CE(0x00345, 0x01ff4), CE(0x00301, 0x003d3), CE(0x00308, 0x003d4),
            CE(0x00308, 0x00407), CE(0x00306, 0x004d0), CE(0x00308, 0x004d2),
            CE(0x00301, 0x00403), CE(0x00300, 0x00400), CE(0x00306, 0x004d6),
            CE(0x00308, 0x00401), CE(0x00306, 0x004c1), CE(0x00308, 0x004dc),
            CE(0x00308, 0x004de), CE(0x00300, 0x0040d), CE(0x00304, 0x004e2),
            CE(0x00306, 0x00419), CE(0x00308, 0x004e4), CE(0x00301, 0x0040c),
            CE(0x00308, 0x004e6), CE(0x00304, 0x004ee), CE(0x00306, 0x0040e),
            CE(0x00308, 0x004f0), CE(0x0030b, 0x004f2), CE(0x00308, 0x004f4),
            CE(0x00308, 0x004f8), CE(0x00308, 0x004ec), CE(0x00306, 0x004d1),
            CE(0x00308, 0x004d3), CE(0x00301, 0x00453), CE(0x00300, 0x00450),
            CE(0x00306, 0x004d7), CE(0x00308, 0x00451), CE(0x00306, 0x004c2),
            CE(0x00308, 0x004dd), CE(0x00308, 0x004df), CE(0x00300, 0x0045d),
            CE(0x00304, 0x004e3), CE(0x00306, 0x00439), CE(0x00308, 0x004e5),
            CE(0x00301, 0x0045c), CE(0x00308, 0x004e7), CE(0x00304, 0x004ef),
            CE(0x00306, 0x0045e), CE(0x00308, 0x004f1), CE(0x0030b, 0x004f3),
            CE(0x00308, 0x004f5), CE(0x00308, 0x004f9), CE(0x00308, 0x004ed),
            CE(0x00308, 0x00457), CE(0x0030f, 0x00476), CE(0x0030f, 0x00477),
            CE(0x00308, 0x004da), CE(0x00308, 0x004db), CE(0x00308, 0x004ea),
            CE(0x00308, 0x004eb), CE(0x00653, 0x00622), CE(0x00654, 0x00623),
            CE(0x00655, 0x00625), CE(0x00654, 0x00624), CE(0x00654, 0x00626),
            CE(0x00654, 0x006c2), CE(0x00654, 0x006d3), CE(0x00654, 0x006c0),
            CE(0x0093c, 0x00929), CE(0x0093c, 0x00931), CE(0x0093c, 0x00934),
            CE(0x009be, 0x009cb), CE(0x009d7, 0x009cc), CE(0x00b3e, 0x00b4b),
            CE(0x00b56, 0x00b48), CE(0x00b57, 0x00b4c), CE(0x00bd7, 0x00b94),
            CE(0x00bbe, 0x00bca), CE(0x00bd7, 0x00bcc), CE(0x00bbe, 0x00bcb),
            CE(0x00c56, 0x00c48), CE(0x00cd5, 0x00cc0), CE(0x00cc2, 0x00cca),
            CE(0x00cd5, 0x00cc7), CE(0x00cd6, 0x00cc8), CE(0x00cd5, 0x00ccb),
            CE(0x00d3e, 0x00d4a), CE(0x00d57, 0x00d4c), CE(0x00d3e, 0x00d4b),
            CE(0x00dca, 0x00dda), CE(0x00dcf, 0x00ddc), CE(0x00ddf, 0x00dde),
            CE(0x00dca, 0x00ddd), CE(0x0102e, 0x01026), CE(0x01b35, 0x01b06),
            CE(0x01b35, 0x01b08), CE(0x01b35, 0x01b0a), CE(0x01b35, 0x01b0c),
            CE(0x01b35, 0x01b0e), CE(0x01b35, 0x01b12), CE(0x01b35, 0x01b3b),
            CE(0x01b35, 0x01b3d), CE(0x01b35, 0x01b40), CE(0x01b35, 0x01b41),
            CE(0x01b35, 0x01b43), CE(0x00304, 0x01e38), CE(0x00304, 0x01e39),
            CE(0x00304, 0x01e5c), CE(0x00304, 0x01e5d), CE(0x00307, 0x01e68),
            CE(0x00307, 0x01e69), CE(0x00302, 0x01eac), CE(0x00306, 0x01eb6),
            CE(0x00302, 0x01ead), CE(0x00306, 0x01eb7), CE(0x00302, 0x01ec6),
            CE(0x00302, 0x01ec7), CE(0x00302, 0x01ed8), CE(0x00302, 0x01ed9),
            CE(0x00300, 0x01f02), CE(0x00301, 0x01f04), CE(0x00342, 0x01f06),
            CE(0x00345, 0x01f80), CE(0x00300, 0x01f03), CE(0x00301, 0x01f05),
            CE(0x00342, 0x01f07), CE(0x00345, 0x01f81), CE(0x00345, 0x01f82),
            CE(0x00345, 0x01f83), CE(0x00345, 0x01f84), CE(0x00345, 0x01f85),
            CE(0x00345, 0x01f86), CE(0x00345, 0x01f87), CE(0x00300, 0x01f0a),
            CE(0x00301, 0x01f0c), CE(0x00342, 0x01f0e), CE(0x00345, 0x01f88),
            CE(0x00300, 0x01f0b), CE(0x00301, 0x01f0d), CE(0x00342, 0x01f0f),
            CE(0x00345, 0x01f89), CE(0x00345, 0x01f8a), CE(0x00345, 0x01f8b),
            CE(0x00345, 0x01f8c), CE(0x00345, 0x01f8d), CE(0x00345, 0x01f8e),
            CE(0x00345, 0x01f8f), CE(0x00300, 0x01f12), CE(0x00301, 0x01f14),
            CE(0x00300, 0x01f13), CE(0x00301, 0x01f15), CE(0x00300, 0x01f1a),
            CE(0x00301, 0x01f1c), CE(0x00300, 0x01f1b), CE(0x00301, 0x01f1d),
            CE(0x00300, 0x01f22), CE(0x00301, 0x01f24), CE(0x00342, 0x01f26),
            CE(0x00345, 0x01f90), CE(0x00300, 0x01f23), CE(0x00301, 0x01f25),
            CE(0x00342, 0x01f27), CE(0x00345, 0x01f91), CE(0x00345, 0x01f92),
            CE(0x00345, 0x01f93), CE(0x00345, 0x01f94), CE(0x00345, 0x01f95),
            CE(0x00345, 0x01f96), CE(0x00345, 0x01f97), CE(0x00300, 0x01f2a),
            CE(0x00301, 0x01f2c), CE(0x00342, 0x01f2e), CE(0x00345, 0x01f98),
            CE(0x00300, 0x01f2b), CE(0x00301, 0x01f2d), CE(0x00342, 0x01f2f),
            CE(0x00345, 0x01f99), CE(0x00345, 0x01f9a), CE(0x00345, 0x01f9b),
            CE(0x00345, 0x01f9c), CE(0x00345, 0x01f9d), CE(0x00345, 0x01f9e),
            CE(0x00345, 0x01f9f), CE(0x00300, 0x01f32), CE(0x00301, 0x01f34),
            CE(0x00342, 0x01f36), CE(0x00300, 0x01f33), CE(0x00301, 0x01f35),
            CE(0x00342, 0x01f37), CE(0x00300, 0x01f3a), CE(0x00301, 0x01f3c),
            CE(0x00342, 0x01f3e), CE(0x00300, 0x01f3b), CE(0x00301, 0x01f3d),
            CE(0x00342, 0x01f3f), CE(0x00300, 0x01f42), CE(0x00301, 0x01f44),
            CE(0x00300, 0x01f43), CE(0x00301, 0x01f45), CE(0x00300, 0x01f4a),
            CE(0x00301, 0x01f4c), CE(0x00300, 0x01f4b), CE(0x00301, 0x01f4d),
            CE(0x00300, 0x01f52), CE(0x00301, 0x01f54), CE(0x00342, 0x01f56),
            CE(0x00300, 0x01f53), CE(0x00301, 0x01f55), CE(0x00342, 0x01f57),
            CE(0x00300, 0x01f5b), CE(0x00301, 0x01f5d), CE(0x00342, 0x01f5f),
            CE(0x00300, 0x01f62), CE(0x00301, 0x01f64), CE(0x00342, 0x01f66),
            CE(0x00345, 0x01fa0), CE(0x00300, 0x01f63), CE(0x00301, 0x01f65),
            CE(0x00342, 0x01f67), CE(0x00345, 0x01fa1), CE(0x00345, 0x01fa2),
            CE(0x00345, 0x01fa3), CE(0x00345, 0x01fa4), CE(0x00345, 0x01fa5),
            CE(0x00345, 0x01fa6), CE(0x00345, 0x01fa7), CE(0x00300, 0x01f6a),
            CE(0x00301, 0x01f6c), CE(0x00342, 0x01f6e), CE(0x00345, 0x01fa8),
            CE(0x00300, 0x01f6b), CE(0x00301, 0x01f6d), CE(0x00342, 0x01f6f),
            CE(0x00345, 0x01fa9), CE(0x00345, 0x01faa), CE(0x00345, 0x01fab),
            CE(0x00345, 0x01fac), CE(0x00345, 0x01fad), CE(0x00345, 0x01fae),
            CE(0x00345, 0x01faf), CE(0x00345, 0x01fb2), CE(0x00345, 0x01fc2),
            CE(0x00345, 0x01ff2), CE(0x00345, 0x01fb7), CE(0x00300, 0x01fcd),
            CE(0x00301, 0x01fce), CE(0x00342, 0x01fcf), CE(0x00345, 0x01fc7),
            CE(0x00345, 0x01ff7), CE(0x00300, 0x01fdd), CE(0x00301, 0x01fde),
            CE(0x00342, 0x01fdf), CE(0x00338, 0x0219a), CE(0x00338, 0x0219b),
            CE(0x00338, 0x021ae), CE(0x00338, 0x021cd), CE(0x00338, 0x021cf),
            CE(0x00338, 0x021ce), CE(0x00338, 0x02204), CE(0x00338, 0x02209),
            CE(0x00338, 0x0220c), CE(0x00338, 0x02224), CE(0x00338, 0x02226),
            CE(0x00338, 0x02241), CE(0x00338, 0x02244), CE(0x00338, 0x02247),
            CE(0x00338, 0x02249), CE(0x00338, 0x0226d), CE(0x00338, 0x02262),
            CE(0x00338, 0x02270), CE(0x00338, 0x02271), CE(0x00338, 0x02274),
            CE(0x00338, 0x02275), CE(0x00338, 0x02278), CE(0x00338, 0x02279),
            CE(0x00338, 0x02280), CE(0x00338, 0x02281), CE(0x00338, 0x022e0),
            CE(0x00338, 0x022e1), CE(0x00338, 0x02284), CE(0x00338, 0x02285),
            CE(0x00338, 0x02288), CE(0x00338, 0x02289), CE(0x00338, 0x022e2),
            CE(0x00338, 0x022e3), CE(0x00338, 0x022ac), CE(0x00338, 0x022ad),
            CE(0x00338, 0x022ae), CE(0x00338, 0x022af), CE(0x00338, 0x022ea),
            CE(0x00338, 0x022eb), CE(0x00338, 0x022ec), CE(0x00338, 0x022ed),
            CE(0x03099, 0x03094), CE(0x03099, 0x0304c), CE(0x03099, 0x0304e),
            CE(0x03099, 0x03050), CE(0x03099, 0x03052), CE(0x03099, 0x03054),
            CE(0x03099, 0x03056), CE(0x03099, 0x03058), CE(0x03099, 0x0305a),
            CE(0x03099, 0x0305c), CE(0x03099, 0x0305e), CE(0x03099, 0x03060),
            CE(0x03099, 0x03062), CE(0x03099, 0x03065), CE(0x03099, 0x03067),
            CE(0x03099, 0x03069), CE(0x03099, 0x03070), CE(0x0309a, 0x03071),
            CE(0x03099, 0x03073), CE(0x0309a, 0x03074), CE(0x03099, 0x03076),
            CE(0x0309a, 0x03077), CE(0x03099, 0x03079), CE(0x0309a, 0x0307a),
            CE(0x03099, 0x0307c), CE(0x0309a, 0x0307d), CE(0x03099, 0x0309e),
            CE(0x03099, 0x030f4), CE(0x03099, 0x030ac), CE(0x03099, 0x030ae),
            CE(0x03099, 0x030b0), CE(0x03099, 0x030b2), CE(0x03099, 0x030b4),
            CE(0x03099, 0x030b6), CE(0x03099, 0x030b8), CE(0x03099, 0x030ba),
            CE(0x03099, 0x030bc), CE(0x03099, 0x030be), CE(0x03099, 0x030c0),
            CE(0x03099, 0x030c2), CE(0x03099, 0x030c5), CE(0x03099, 0x030c7),
            CE(0x03099, 0x030c9), CE(0x03099, 0x030d0), CE(0x0309a, 0x030d1),
            CE(0x03099, 0x030d3), CE(0x0309a, 0x030d4), CE(0x03099, 0x030d6),
            CE(0x0309a, 0x030d7), CE(0x03099, 0x030d9), CE(0x0309a, 0x030da),
            CE(0x03099, 0x030dc), CE(0x0309a, 0x030dd), CE(0x03099, 0x030f7),
            CE(0x03099, 0x030f8), CE(0x03099, 0x030f9), CE(0x03099, 0x030fa),
            CE(0x03099, 0x030fe), CE(0x110ba, 0x1109a), CE(0x110ba, 0x1109c),
            CE(0x110ba, 0x110ab), CE(0x11127, 0x1112e), CE(0x11127, 0x1112f),
        ];
        return t;
    }

}

static if (size_t.sizeof == 4)
{
    //6976 bytes
    enum combiningClassTrieEntries = TrieEntry!(ubyte, 8, 7, 6)([0x0, 0x40,
            0x240], [0x100, 0x400, 0x1240], [0x2020100, 0x4020302, 0x2020205,
            0x2060202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202, 0x2020202,
            0x2020202, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x20001, 0x0, 0x0, 0x3,
            0x0, 0x50004, 0x70006, 0x80000, 0xa0009, 0xb0000, 0xd000c, 0xe0000,
            0x10000f, 0x11000f, 0x11000f, 0x11000f, 0x11000f, 0x110000,
            0x120000, 0x11000f, 0x110000, 0x130000, 0x150014, 0x170016,
            0x190018, 0x1b001a, 0x1c, 0x1d, 0x0, 0x0, 0x0, 0x0, 0x1e0000, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f, 0x200000, 0x0, 0x21, 0x22, 0x0,
            0x240023, 0x0, 0x260025, 0x280027, 0x29, 0x2a0000, 0x0, 0x2b0000,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x2c0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x2d0000, 0x2e0000, 0x2f0000, 0x0, 0x0, 0x0,
            0x0, 0x30, 0x31, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x320000, 0x340033, 0x0, 0x0, 0x35,
            0x360000, 0x380037, 0x3a0039, 0x0, 0x3c003b, 0x0, 0x3d0000, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x3e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3f, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x400000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x41, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x350000, 0x42, 0x43, 0x3a0000, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x44, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x450000, 0x46,
            0x470000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6,
            0xe6e6e6e6, 0xe6e6e6e6, 0xdcdce8e6, 0xd8e8dcdc, 0xdcdcdcdc,
            0xdccacadc, 0xcadcdcdc, 0xdcdcdcca, 0xdcdcdcdc, 0xdcdcdcdc,
            0x1010101, 0xdcdcdc01, 0xe6e6e6dc, 0xe6e6e6e6, 0xdce6f0e6,
            0xe6e6dcdc, 0xdcdce6, 0xdce6e6e6, 0xe6dcdcdc, 0xe6dcdce8,
            0xe9eaeae9, 0xe6e9eaea, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6, 0x0,
            0x0, 0x0, 0x0, 0xe6000000, 0xe6e6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xe6e6dc00, 0xe6dce6e6, 0xdcdee6e6, 0xe6e6e6e6, 0xdcdce6e6,
            0xdcdcdcdc, 0xe6dce6e6, 0xe6e4dee6, 0xd0c0b0a, 0x11100f0e,
            0x14131312, 0x17001615, 0x191800, 0x1200dce6, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0xe6e6e6e6, 0xe6e6e6e6, 0x201f1e, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1b000000, 0x1f1e1d1c, 0xe6222120,
            0xe6dcdce6, 0xe6e6e6e6, 0xdce6e6dc, 0x0, 0x0, 0x0, 0x0, 0x23, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e60000, 0xe6e6e6e6,
            0xe60000e6, 0xdce6e6e6, 0xe60000e6, 0xe6dc00e6, 0xdce6, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2400, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0xe6e6dce6, 0xdce6e6dc, 0xdce6dcdc, 0xe6dce6dc, 0xe6dce6e6,
            0xe6dce6dc, 0xe6e6dc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0xe6000000, 0xe6e6e6e6, 0xe6dce6e6, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xe6e60000, 0xe600e6e6, 0xe6e6e6e6, 0xe6e6e6e6,
            0xe6e6e600, 0xe6e6e600, 0xe6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xdcdcdc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6dce6e6,
            0xe6e6dce6, 0xdcdcdce6, 0xe61d1c1b, 0xe6dce6e6, 0xe6dcdce6,
            0xe6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x7, 0x0, 0x0, 0x0, 0x900, 0xe6dce600, 0xe6,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x900, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x900, 0x0, 0x5b5400, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x90000, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x96767, 0x0,
            0x0, 0x0, 0x6b6b6b6b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x7676, 0x0, 0x0, 0x0, 0x7a7a7a7a, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0xdcdc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xdc00dc00, 0xd800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x828100, 0x84, 0x82820000, 0x8282, 0xe6e60082,
            0xe6e60009, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0xdc0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7000000, 0x90900, 0x0, 0x0,
            0x0, 0x0, 0xdc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e600, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x90000, 0x0, 0x0, 0xe600, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe400,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xdce6de00, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0xe6000000, 0xdc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9, 0x0, 0x0, 0x0, 0x0,
            0xe6e6e600, 0xe6e6e6e6, 0xdc0000e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0x0, 0x0, 0x0, 0x9, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6000000, 0xe6e6e6dc,
            0xe6e6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x9090000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x70000, 0x0, 0x0, 0x9090000, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x7000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e6, 0xdcdcdc01,
            0xe6e6dcdc, 0xdcdcdcdc, 0x10100e6, 0x1010101, 0x1, 0xdc00, 0x0,
            0xe6, 0x0, 0x0, 0xe6dce6e6, 0xe6e6e6e6, 0xe6dce6e6, 0xdcd6eae6,
            0xe6e6e6ca, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6,
            0xe6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0xdce6dce9, 0x0, 0x0, 0x0, 0x0,
            0x101e6e6, 0xe6e6e6e6, 0xe6010101, 0xe6, 0xe600, 0xe6010100,
            0x101e6dc, 0xdcdcdcdc, 0xe6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6000000, 0xe6e6, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x9000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6,
            0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0xe4da0000, 0xe0e0dee8, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80800, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0xe6000000, 0x0, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6000000, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0xe6e6, 0x0, 0x0, 0x0, 0x0, 0x90000, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e6e6e6, 0xe6e6e6e6, 0xe6e6e6e6,
            0xe6e6e6e6, 0xe6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0xdc000000, 0xdcdc, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x9000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x7000000, 0x0, 0x0, 0x0, 0x9, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xe6e600e6,
            0xe60000dc, 0xe6, 0xe6e60000, 0xe600, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x90000, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x900, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1a0000, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0xe6e6e6e6, 0xe6e6e6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xdc00,
            0x0, 0x0, 0x0, 0xe600dc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0xdc01e6, 0x9000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x70900, 0x0, 0xe6e6e6, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x9000000, 0x9,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x7090000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x1d8d800, 0x101, 0xd8d8e200, 0xd8d8d8, 0x0, 0xdc000000,
            0xdcdcdcdc, 0xdcdcdc, 0xe6e6e600, 0xdcdce6e6, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0xe6e60000, 0xe6e6, 0x0, 0x0, 0x0, 0x0, 0xe6e60000,
            0xe6, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
            0x0, 0x0, 0x0, 0x0, 0x0]);
    enum composeIdxMask = (1 << 11) - 1, composeCntShift = 11;
    enum compositionJumpTrieEntries = TrieEntry!(ushort, 12, 9)([0x0, 0x800],
            [0x1000, 0x2000], [0x10000, 0x30002, 0x50004, 0x70006, 0x70008,
            0x70007, 0x90007, 0xa0007, 0xc000b, 0x70007, 0x70007, 0x70007,
            0x7000d, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x7000e, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007,
            0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0x70007, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8010800,
            0xffff0802, 0x8003ffff, 0x28161813, 0x8821301b, 0x38330832,
            0x7841383a, 0x28510850, 0x185c3056, 0x8068485f, 0xffff1078,
            0x3882407a, 0x98903889, 0x30a510a3, 0x48ad10ab, 0xffff30b6,
            0xffffffff, 0xffffffff, 0x80bcffff, 0x28cf18cc, 0x88da30d4,
            0x38ec08eb, 0x70fb40f3, 0x290b1109, 0x19163110, 0x81224919,
            0xffff1132, 0x393c4134, 0x994b4143, 0x3960115e, 0x51691167,
            0xffff3173, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff1979, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffff217c, 0x9810980, 0x9841182, 0xffffffff,
            0xffff2185, 0xffffffff, 0x989ffff, 0xffffffff, 0xffffffff,
            0x198e218a, 0xffff0991, 0xffff0992, 0xffffffff, 0xffff2193,
            0xffffffff, 0xffffffff, 0xffff2197, 0x99c099b, 0x99f119d,
            0xffffffff, 0xffff21a0, 0xffffffff, 0x9a4ffff, 0xffffffff,
            0xffffffff, 0x19a921a5, 0xffff09ac, 0xffff09ad, 0xffffffff,
            0xffff21ae, 0xffffffff, 0xffffffff, 0x21b621b2, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0x11bc11ba, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0x11c011be, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0x9c309c2, 0xffffffff, 0xffffffff,
            0x9c509c4, 0xffffffff, 0xffffffff, 0xffffffff, 0x9c709c6,
            0x9c909c8, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0x9caffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0x29d029cb, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x29d5ffff,
            0xffff29da, 0xffffffff, 0xffffffff, 0x9dfffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x9e109e0,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x9e309e2,
            0x9e509e4, 0xffffffff, 0xffffffff, 0x9e709e6, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff09e8, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0x39e9ffff, 0xffffffff, 0x21f0ffff, 0x29f4ffff, 0x39f9ffff,
            0xffffffff, 0xffffffff, 0x2200ffff, 0xa04ffff, 0xffffffff,
            0x3205ffff, 0xffffffff, 0x2a0bffff, 0xffffffff, 0xffff0a10,
            0xffff0a11, 0x4212ffff, 0xffffffff, 0x221affff, 0x321effff,
            0x4224ffff, 0xffffffff, 0xffffffff, 0x222cffff, 0x1230ffff,
            0xffffffff, 0x4232ffff, 0xffffffff, 0x323affff, 0x1a431a40,
            0xffffffff, 0xffff0a46, 0xffffffff, 0xffff1247, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0a49,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff124a,
            0xa4cffff, 0x1a4dffff, 0xa521250, 0xffff2253, 0xffff0a57,
            0xffffffff, 0xffff0a58, 0xffffffff, 0x2259ffff, 0xffffffff,
            0xa5dffff, 0xffffffff, 0xa5effff, 0xa5fffff, 0xffffffff,
            0xffff1260, 0xa62ffff, 0x1a63ffff, 0xa681266, 0xffff2269,
            0xffff0a6d, 0xffffffff, 0xffff0a6e, 0xffffffff, 0x226fffff,
            0xffffffff, 0xa73ffff, 0xffffffff, 0xa74ffff, 0xa75ffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0a76,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa780a77,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa7a0a79,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xa7c0a7b, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0x1a7dffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff0a80, 0xffff0a81,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xa82ffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffff0a83, 0xa84ffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffff0a85, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffff0a86, 0xa87ffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0x1288ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0x1a8affff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffff0a8d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xa90128e, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffff0a91, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xa92ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff1a93,
            0xffffffff, 0xffff0a96, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xa991297, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0x1a9affff, 0xffffffff, 0xffff0a9d, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xa9effff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xa9fffff, 0xaa0ffff,
            0xaa1ffff, 0xaa2ffff, 0xaa3ffff, 0xffffffff, 0xaa4ffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0aa5,
            0xffff0aa6, 0xaa80aa7, 0xffffffff, 0xffff0aa9, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xaab0aaa, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xaad0aac, 0xffffffff,
            0xffffffff, 0xffffffff, 0xaaf0aae, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0x12b212b0, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xab50ab4, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xab70ab6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0x22bc22b8, 0xac10ac0, 0xac30ac2, 0xac50ac4,
            0x22ca22c6, 0xacf0ace, 0xad10ad0, 0xad30ad2, 0x12d612d4,
            0xffffffff, 0xffffffff, 0xffffffff, 0x12da12d8, 0xffffffff,
            0xffffffff, 0xffffffff, 0x22e022dc, 0xae50ae4, 0xae70ae6,
            0xae90ae8, 0x22ee22ea, 0xaf30af2, 0xaf50af4, 0xaf70af6, 0x1afb1af8,
            0xffffffff, 0xffffffff, 0xffffffff, 0x1b011afe, 0xffffffff,
            0xffffffff, 0xffffffff, 0x13061304, 0xffffffff, 0xffffffff,
            0xffffffff, 0x130a1308, 0xffffffff, 0xffffffff, 0xffffffff,
            0x1b0f1b0c, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b12ffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0x23192315, 0xb1e0b1d,
            0xb200b1f, 0xb220b21, 0x23272323, 0xb2c0b2b, 0xb2e0b2d, 0xb300b2f,
            0xffff0b31, 0xffffffff, 0xffff0b32, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffff0b33, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffff0b34, 0xffffffff, 0xffffffff, 0xffffffff, 0x1b35ffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b38, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffff0b39, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffff1b3a, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffff0b3d, 0xffff0b3e, 0xffff0b3f,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b40,
            0xffff0b41, 0xffff0b42, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb43ffff,
            0xffffffff, 0xffffffff, 0xffff0b44, 0xb45ffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xb46ffff, 0xb47ffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b48, 0xffffffff,
            0xffffffff, 0xb49ffff, 0xb4affff, 0xffffffff, 0xffff0b4b,
            0xffffffff, 0xb4cffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xb4dffff, 0xffffffff, 0xb4f0b4e, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xb510b50, 0xffffffff, 0xb530b52, 0xffffffff, 0xb550b54, 0xb570b56,
            0xffffffff, 0xffffffff, 0xb590b58, 0xffffffff, 0xb5b0b5a,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb5cffff,
            0xffff0b5d, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b5e, 0xffffffff,
            0xffffffff, 0xb600b5f, 0xb61ffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xb630b62, 0xb650b64, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b66, 0xffffffff,
            0xb67ffff, 0xb68ffff, 0xb69ffff, 0xb6affff, 0xb6bffff, 0xb6cffff,
            0xb6dffff, 0xb6effff, 0xb6fffff, 0xb70ffff, 0xb71ffff, 0xb72ffff,
            0xffffffff, 0xffff0b73, 0xffff0b74, 0xffff0b75, 0xffffffff,
            0xffffffff, 0x1376ffff, 0xffffffff, 0xffff1378, 0x137affff,
            0xffffffff, 0xffff137c, 0x137effff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb80ffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0b81,
            0xffffffff, 0xb82ffff, 0xb83ffff, 0xb84ffff, 0xb85ffff, 0xb86ffff,
            0xb87ffff, 0xb88ffff, 0xb89ffff, 0xb8affff, 0xb8bffff, 0xb8cffff,
            0xb8dffff, 0xffffffff, 0xffff0b8e, 0xffff0b8f, 0xffff0b90,
            0xffffffff, 0xffffffff, 0x1391ffff, 0xffffffff, 0xffff1393,
            0x1395ffff, 0xffffffff, 0xffff1397, 0x1399ffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xb9bffff, 0xb9d0b9c,
            0xffff0b9e, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xb9fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xba0ffff, 0xba1ffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xba2ffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xba3ffff, 0xffff0ba4, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
            0xffffffff]);
    @property immutable(CompEntry[]) compositionTable()
    {
        alias CE = CompEntry;
        static immutable CE[] t = [
            CE(0x00338, 0x0226e), CE(0x00338, 0x02260), CE(0x00338, 0x0226f),
            CE(0x00300, 0x000c0), CE(0x00301, 0x000c1), CE(0x00302, 0x000c2),
            CE(0x00303, 0x000c3), CE(0x00304, 0x00100), CE(0x00306, 0x00102),
            CE(0x00307, 0x00226), CE(0x00308, 0x000c4), CE(0x00309, 0x01ea2),
            CE(0x0030a, 0x000c5), CE(0x0030c, 0x001cd), CE(0x0030f, 0x00200),
            CE(0x00311, 0x00202), CE(0x00323, 0x01ea0), CE(0x00325, 0x01e00),
            CE(0x00328, 0x00104), CE(0x00307, 0x01e02), CE(0x00323, 0x01e04),
            CE(0x00331, 0x01e06), CE(0x00301, 0x00106), CE(0x00302, 0x00108),
            CE(0x00307, 0x0010a), CE(0x0030c, 0x0010c), CE(0x00327, 0x000c7),
            CE(0x00307, 0x01e0a), CE(0x0030c, 0x0010e), CE(0x00323, 0x01e0c),
            CE(0x00327, 0x01e10), CE(0x0032d, 0x01e12), CE(0x00331, 0x01e0e),
            CE(0x00300, 0x000c8), CE(0x00301, 0x000c9), CE(0x00302, 0x000ca),
            CE(0x00303, 0x01ebc), CE(0x00304, 0x00112), CE(0x00306, 0x00114),
            CE(0x00307, 0x00116), CE(0x00308, 0x000cb), CE(0x00309, 0x01eba),
            CE(0x0030c, 0x0011a), CE(0x0030f, 0x00204), CE(0x00311, 0x00206),
            CE(0x00323, 0x01eb8), CE(0x00327, 0x00228), CE(0x00328, 0x00118),
            CE(0x0032d, 0x01e18), CE(0x00330, 0x01e1a), CE(0x00307, 0x01e1e),
            CE(0x00301, 0x001f4), CE(0x00302, 0x0011c), CE(0x00304, 0x01e20),
            CE(0x00306, 0x0011e), CE(0x00307, 0x00120), CE(0x0030c, 0x001e6),
            CE(0x00327, 0x00122), CE(0x00302, 0x00124), CE(0x00307, 0x01e22),
            CE(0x00308, 0x01e26), CE(0x0030c, 0x0021e), CE(0x00323, 0x01e24),
            CE(0x00327, 0x01e28), CE(0x0032e, 0x01e2a), CE(0x00300, 0x000cc),
            CE(0x00301, 0x000cd), CE(0x00302, 0x000ce), CE(0x00303, 0x00128),
            CE(0x00304, 0x0012a), CE(0x00306, 0x0012c), CE(0x00307, 0x00130),
            CE(0x00308, 0x000cf), CE(0x00309, 0x01ec8), CE(0x0030c, 0x001cf),
            CE(0x0030f, 0x00208), CE(0x00311, 0x0020a), CE(0x00323, 0x01eca),
            CE(0x00328, 0x0012e), CE(0x00330, 0x01e2c), CE(0x00302, 0x00134),
            CE(0x00301, 0x01e30), CE(0x0030c, 0x001e8), CE(0x00323, 0x01e32),
            CE(0x00327, 0x00136), CE(0x00331, 0x01e34), CE(0x00301, 0x00139),
            CE(0x0030c, 0x0013d), CE(0x00323, 0x01e36), CE(0x00327, 0x0013b),
            CE(0x0032d, 0x01e3c), CE(0x00331, 0x01e3a), CE(0x00301, 0x01e3e),
            CE(0x00307, 0x01e40), CE(0x00323, 0x01e42), CE(0x00300, 0x001f8),
            CE(0x00301, 0x00143), CE(0x00303, 0x000d1), CE(0x00307, 0x01e44),
            CE(0x0030c, 0x00147), CE(0x00323, 0x01e46), CE(0x00327, 0x00145),
            CE(0x0032d, 0x01e4a), CE(0x00331, 0x01e48), CE(0x00300, 0x000d2),
            CE(0x00301, 0x000d3), CE(0x00302, 0x000d4), CE(0x00303, 0x000d5),
            CE(0x00304, 0x0014c), CE(0x00306, 0x0014e), CE(0x00307, 0x0022e),
            CE(0x00308, 0x000d6), CE(0x00309, 0x01ece), CE(0x0030b, 0x00150),
            CE(0x0030c, 0x001d1), CE(0x0030f, 0x0020c), CE(0x00311, 0x0020e),
            CE(0x0031b, 0x001a0), CE(0x00323, 0x01ecc), CE(0x00328, 0x001ea),
            CE(0x00301, 0x01e54), CE(0x00307, 0x01e56), CE(0x00301, 0x00154),
            CE(0x00307, 0x01e58), CE(0x0030c, 0x00158), CE(0x0030f, 0x00210),
            CE(0x00311, 0x00212), CE(0x00323, 0x01e5a), CE(0x00327, 0x00156),
            CE(0x00331, 0x01e5e), CE(0x00301, 0x0015a), CE(0x00302, 0x0015c),
            CE(0x00307, 0x01e60), CE(0x0030c, 0x00160), CE(0x00323, 0x01e62),
            CE(0x00326, 0x00218), CE(0x00327, 0x0015e), CE(0x00307, 0x01e6a),
            CE(0x0030c, 0x00164), CE(0x00323, 0x01e6c), CE(0x00326, 0x0021a),
            CE(0x00327, 0x00162), CE(0x0032d, 0x01e70), CE(0x00331, 0x01e6e),
            CE(0x00300, 0x000d9), CE(0x00301, 0x000da), CE(0x00302, 0x000db),
            CE(0x00303, 0x00168), CE(0x00304, 0x0016a), CE(0x00306, 0x0016c),
            CE(0x00308, 0x000dc), CE(0x00309, 0x01ee6), CE(0x0030a, 0x0016e),
            CE(0x0030b, 0x00170), CE(0x0030c, 0x001d3), CE(0x0030f, 0x00214),
            CE(0x00311, 0x00216), CE(0x0031b, 0x001af), CE(0x00323, 0x01ee4),
            CE(0x00324, 0x01e72), CE(0x00328, 0x00172), CE(0x0032d, 0x01e76),
            CE(0x00330, 0x01e74), CE(0x00303, 0x01e7c), CE(0x00323, 0x01e7e),
            CE(0x00300, 0x01e80), CE(0x00301, 0x01e82), CE(0x00302, 0x00174),
            CE(0x00307, 0x01e86), CE(0x00308, 0x01e84), CE(0x00323, 0x01e88),
            CE(0x00307, 0x01e8a), CE(0x00308, 0x01e8c), CE(0x00300, 0x01ef2),
            CE(0x00301, 0x000dd), CE(0x00302, 0x00176), CE(0x00303, 0x01ef8),
            CE(0x00304, 0x00232), CE(0x00307, 0x01e8e), CE(0x00308, 0x00178),
            CE(0x00309, 0x01ef6), CE(0x00323, 0x01ef4), CE(0x00301, 0x00179),
            CE(0x00302, 0x01e90), CE(0x00307, 0x0017b), CE(0x0030c, 0x0017d),
            CE(0x00323, 0x01e92), CE(0x00331, 0x01e94), CE(0x00300, 0x000e0),
            CE(0x00301, 0x000e1), CE(0x00302, 0x000e2), CE(0x00303, 0x000e3),
            CE(0x00304, 0x00101), CE(0x00306, 0x00103), CE(0x00307, 0x00227),
            CE(0x00308, 0x000e4), CE(0x00309, 0x01ea3), CE(0x0030a, 0x000e5),
            CE(0x0030c, 0x001ce), CE(0x0030f, 0x00201), CE(0x00311, 0x00203),
            CE(0x00323, 0x01ea1), CE(0x00325, 0x01e01), CE(0x00328, 0x00105),
            CE(0x00307, 0x01e03), CE(0x00323, 0x01e05), CE(0x00331, 0x01e07),
            CE(0x00301, 0x00107), CE(0x00302, 0x00109), CE(0x00307, 0x0010b),
            CE(0x0030c, 0x0010d), CE(0x00327, 0x000e7), CE(0x00307, 0x01e0b),
            CE(0x0030c, 0x0010f), CE(0x00323, 0x01e0d), CE(0x00327, 0x01e11),
            CE(0x0032d, 0x01e13), CE(0x00331, 0x01e0f), CE(0x00300, 0x000e8),
            CE(0x00301, 0x000e9), CE(0x00302, 0x000ea), CE(0x00303, 0x01ebd),
            CE(0x00304, 0x00113), CE(0x00306, 0x00115), CE(0x00307, 0x00117),
            CE(0x00308, 0x000eb), CE(0x00309, 0x01ebb), CE(0x0030c, 0x0011b),
            CE(0x0030f, 0x00205), CE(0x00311, 0x00207), CE(0x00323, 0x01eb9),
            CE(0x00327, 0x00229), CE(0x00328, 0x00119), CE(0x0032d, 0x01e19),
            CE(0x00330, 0x01e1b), CE(0x00307, 0x01e1f), CE(0x00301, 0x001f5),
            CE(0x00302, 0x0011d), CE(0x00304, 0x01e21), CE(0x00306, 0x0011f),
            CE(0x00307, 0x00121), CE(0x0030c, 0x001e7), CE(0x00327, 0x00123),
            CE(0x00302, 0x00125), CE(0x00307, 0x01e23), CE(0x00308, 0x01e27),
            CE(0x0030c, 0x0021f), CE(0x00323, 0x01e25), CE(0x00327, 0x01e29),
            CE(0x0032e, 0x01e2b), CE(0x00331, 0x01e96), CE(0x00300, 0x000ec),
            CE(0x00301, 0x000ed), CE(0x00302, 0x000ee), CE(0x00303, 0x00129),
            CE(0x00304, 0x0012b), CE(0x00306, 0x0012d), CE(0x00308, 0x000ef),
            CE(0x00309, 0x01ec9), CE(0x0030c, 0x001d0), CE(0x0030f, 0x00209),
            CE(0x00311, 0x0020b), CE(0x00323, 0x01ecb), CE(0x00328, 0x0012f),
            CE(0x00330, 0x01e2d), CE(0x00302, 0x00135), CE(0x0030c, 0x001f0),
            CE(0x00301, 0x01e31), CE(0x0030c, 0x001e9), CE(0x00323, 0x01e33),
            CE(0x00327, 0x00137), CE(0x00331, 0x01e35), CE(0x00301, 0x0013a),
            CE(0x0030c, 0x0013e), CE(0x00323, 0x01e37), CE(0x00327, 0x0013c),
            CE(0x0032d, 0x01e3d), CE(0x00331, 0x01e3b), CE(0x00301, 0x01e3f),
            CE(0x00307, 0x01e41), CE(0x00323, 0x01e43), CE(0x00300, 0x001f9),
            CE(0x00301, 0x00144), CE(0x00303, 0x000f1), CE(0x00307, 0x01e45),
            CE(0x0030c, 0x00148), CE(0x00323, 0x01e47), CE(0x00327, 0x00146),
            CE(0x0032d, 0x01e4b), CE(0x00331, 0x01e49), CE(0x00300, 0x000f2),
            CE(0x00301, 0x000f3), CE(0x00302, 0x000f4), CE(0x00303, 0x000f5),
            CE(0x00304, 0x0014d), CE(0x00306, 0x0014f), CE(0x00307, 0x0022f),
            CE(0x00308, 0x000f6), CE(0x00309, 0x01ecf), CE(0x0030b, 0x00151),
            CE(0x0030c, 0x001d2), CE(0x0030f, 0x0020d), CE(0x00311, 0x0020f),
            CE(0x0031b, 0x001a1), CE(0x00323, 0x01ecd), CE(0x00328, 0x001eb),
            CE(0x00301, 0x01e55), CE(0x00307, 0x01e57), CE(0x00301, 0x00155),
            CE(0x00307, 0x01e59), CE(0x0030c, 0x00159), CE(0x0030f, 0x00211),
            CE(0x00311, 0x00213), CE(0x00323, 0x01e5b), CE(0x00327, 0x00157),
            CE(0x00331, 0x01e5f), CE(0x00301, 0x0015b), CE(0x00302, 0x0015d),
            CE(0x00307, 0x01e61), CE(0x0030c, 0x00161), CE(0x00323, 0x01e63),
            CE(0x00326, 0x00219), CE(0x00327, 0x0015f), CE(0x00307, 0x01e6b),
            CE(0x00308, 0x01e97), CE(0x0030c, 0x00165), CE(0x00323, 0x01e6d),
            CE(0x00326, 0x0021b), CE(0x00327, 0x00163), CE(0x0032d, 0x01e71),
            CE(0x00331, 0x01e6f), CE(0x00300, 0x000f9), CE(0x00301, 0x000fa),
            CE(0x00302, 0x000fb), CE(0x00303, 0x00169), CE(0x00304, 0x0016b),
            CE(0x00306, 0x0016d), CE(0x00308, 0x000fc), CE(0x00309, 0x01ee7),
            CE(0x0030a, 0x0016f), CE(0x0030b, 0x00171), CE(0x0030c, 0x001d4),
            CE(0x0030f, 0x00215), CE(0x00311, 0x00217), CE(0x0031b, 0x001b0),
            CE(0x00323, 0x01ee5), CE(0x00324, 0x01e73), CE(0x00328, 0x00173),
            CE(0x0032d, 0x01e77), CE(0x00330, 0x01e75), CE(0x00303, 0x01e7d),
            CE(0x00323, 0x01e7f), CE(0x00300, 0x01e81), CE(0x00301, 0x01e83),
            CE(0x00302, 0x00175), CE(0x00307, 0x01e87), CE(0x00308, 0x01e85),
            CE(0x0030a, 0x01e98), CE(0x00323, 0x01e89), CE(0x00307, 0x01e8b),
            CE(0x00308, 0x01e8d), CE(0x00300, 0x01ef3), CE(0x00301, 0x000fd),
            CE(0x00302, 0x00177), CE(0x00303, 0x01ef9), CE(0x00304, 0x00233),
            CE(0x00307, 0x01e8f), CE(0x00308, 0x000ff), CE(0x00309, 0x01ef7),
            CE(0x0030a, 0x01e99), CE(0x00323, 0x01ef5), CE(0x00301, 0x0017a),
            CE(0x00302, 0x01e91), CE(0x00307, 0x0017c), CE(0x0030c, 0x0017e),
            CE(0x00323, 0x01e93), CE(0x00331, 0x01e95), CE(0x00300, 0x01fed),
            CE(0x00301, 0x00385), CE(0x00342, 0x01fc1), CE(0x00300, 0x01ea6),
            CE(0x00301, 0x01ea4), CE(0x00303, 0x01eaa), CE(0x00309, 0x01ea8),
            CE(0x00304, 0x001de), CE(0x00301, 0x001fa), CE(0x00301, 0x001fc),
            CE(0x00304, 0x001e2), CE(0x00301, 0x01e08), CE(0x00300, 0x01ec0),
            CE(0x00301, 0x01ebe), CE(0x00303, 0x01ec4), CE(0x00309, 0x01ec2),
            CE(0x00301, 0x01e2e), CE(0x00300, 0x01ed2), CE(0x00301, 0x01ed0),
            CE(0x00303, 0x01ed6), CE(0x00309, 0x01ed4), CE(0x00301, 0x01e4c),
            CE(0x00304, 0x0022c), CE(0x00308, 0x01e4e), CE(0x00304, 0x0022a),
            CE(0x00301, 0x001fe), CE(0x00300, 0x001db), CE(0x00301, 0x001d7),
            CE(0x00304, 0x001d5), CE(0x0030c, 0x001d9), CE(0x00300, 0x01ea7),
            CE(0x00301, 0x01ea5), CE(0x00303, 0x01eab), CE(0x00309, 0x01ea9),
            CE(0x00304, 0x001df), CE(0x00301, 0x001fb), CE(0x00301, 0x001fd),
            CE(0x00304, 0x001e3), CE(0x00301, 0x01e09), CE(0x00300, 0x01ec1),
            CE(0x00301, 0x01ebf), CE(0x00303, 0x01ec5), CE(0x00309, 0x01ec3),
            CE(0x00301, 0x01e2f), CE(0x00300, 0x01ed3), CE(0x00301, 0x01ed1),
            CE(0x00303, 0x01ed7), CE(0x00309, 0x01ed5), CE(0x00301, 0x01e4d),
            CE(0x00304, 0x0022d), CE(0x00308, 0x01e4f), CE(0x00304, 0x0022b),
            CE(0x00301, 0x001ff), CE(0x00300, 0x001dc), CE(0x00301, 0x001d8),
            CE(0x00304, 0x001d6), CE(0x0030c, 0x001da), CE(0x00300, 0x01eb0),
            CE(0x00301, 0x01eae), CE(0x00303, 0x01eb4), CE(0x00309, 0x01eb2),
            CE(0x00300, 0x01eb1), CE(0x00301, 0x01eaf), CE(0x00303, 0x01eb5),
            CE(0x00309, 0x01eb3), CE(0x00300, 0x01e14), CE(0x00301, 0x01e16),
            CE(0x00300, 0x01e15), CE(0x00301, 0x01e17), CE(0x00300, 0x01e50),
            CE(0x00301, 0x01e52), CE(0x00300, 0x01e51), CE(0x00301, 0x01e53),
            CE(0x00307, 0x01e64), CE(0x00307, 0x01e65), CE(0x00307, 0x01e66),
            CE(0x00307, 0x01e67), CE(0x00301, 0x01e78), CE(0x00301, 0x01e79),
            CE(0x00308, 0x01e7a), CE(0x00308, 0x01e7b), CE(0x00307, 0x01e9b),
            CE(0x00300, 0x01edc), CE(0x00301, 0x01eda), CE(0x00303, 0x01ee0),
            CE(0x00309, 0x01ede), CE(0x00323, 0x01ee2), CE(0x00300, 0x01edd),
            CE(0x00301, 0x01edb), CE(0x00303, 0x01ee1), CE(0x00309, 0x01edf),
            CE(0x00323, 0x01ee3), CE(0x00300, 0x01eea), CE(0x00301, 0x01ee8),
            CE(0x00303, 0x01eee), CE(0x00309, 0x01eec), CE(0x00323, 0x01ef0),
            CE(0x00300, 0x01eeb), CE(0x00301, 0x01ee9), CE(0x00303, 0x01eef),
            CE(0x00309, 0x01eed), CE(0x00323, 0x01ef1), CE(0x0030c, 0x001ee),
            CE(0x00304, 0x001ec), CE(0x00304, 0x001ed), CE(0x00304, 0x001e0),
            CE(0x00304, 0x001e1), CE(0x00306, 0x01e1c), CE(0x00306, 0x01e1d),
            CE(0x00304, 0x00230), CE(0x00304, 0x00231), CE(0x0030c, 0x001ef),
            CE(0x00300, 0x01fba), CE(0x00301, 0x00386), CE(0x00304, 0x01fb9),
            CE(0x00306, 0x01fb8), CE(0x00313, 0x01f08), CE(0x00314, 0x01f09),
            CE(0x00345, 0x01fbc), CE(0x00300, 0x01fc8), CE(0x00301, 0x00388),
            CE(0x00313, 0x01f18), CE(0x00314, 0x01f19), CE(0x00300, 0x01fca),
            CE(0x00301, 0x00389), CE(0x00313, 0x01f28), CE(0x00314, 0x01f29),
            CE(0x00345, 0x01fcc), CE(0x00300, 0x01fda), CE(0x00301, 0x0038a),
            CE(0x00304, 0x01fd9), CE(0x00306, 0x01fd8), CE(0x00308, 0x003aa),
            CE(0x00313, 0x01f38), CE(0x00314, 0x01f39), CE(0x00300, 0x01ff8),
            CE(0x00301, 0x0038c), CE(0x00313, 0x01f48), CE(0x00314, 0x01f49),
            CE(0x00314, 0x01fec), CE(0x00300, 0x01fea), CE(0x00301, 0x0038e),
            CE(0x00304, 0x01fe9), CE(0x00306, 0x01fe8), CE(0x00308, 0x003ab),
            CE(0x00314, 0x01f59), CE(0x00300, 0x01ffa), CE(0x00301, 0x0038f),
            CE(0x00313, 0x01f68), CE(0x00314, 0x01f69), CE(0x00345, 0x01ffc),
            CE(0x00345, 0x01fb4), CE(0x00345, 0x01fc4), CE(0x00300, 0x01f70),
            CE(0x00301, 0x003ac), CE(0x00304, 0x01fb1), CE(0x00306, 0x01fb0),
            CE(0x00313, 0x01f00), CE(0x00314, 0x01f01), CE(0x00342, 0x01fb6),
            CE(0x00345, 0x01fb3), CE(0x00300, 0x01f72), CE(0x00301, 0x003ad),
            CE(0x00313, 0x01f10), CE(0x00314, 0x01f11), CE(0x00300, 0x01f74),
            CE(0x00301, 0x003ae), CE(0x00313, 0x01f20), CE(0x00314, 0x01f21),
            CE(0x00342, 0x01fc6), CE(0x00345, 0x01fc3), CE(0x00300, 0x01f76),
            CE(0x00301, 0x003af), CE(0x00304, 0x01fd1), CE(0x00306, 0x01fd0),
            CE(0x00308, 0x003ca), CE(0x00313, 0x01f30), CE(0x00314, 0x01f31),
            CE(0x00342, 0x01fd6), CE(0x00300, 0x01f78), CE(0x00301, 0x003cc),
            CE(0x00313, 0x01f40), CE(0x00314, 0x01f41), CE(0x00313, 0x01fe4),
            CE(0x00314, 0x01fe5), CE(0x00300, 0x01f7a), CE(0x00301, 0x003cd),
            CE(0x00304, 0x01fe1), CE(0x00306, 0x01fe0), CE(0x00308, 0x003cb),
            CE(0x00313, 0x01f50), CE(0x00314, 0x01f51), CE(0x00342, 0x01fe6),
            CE(0x00300, 0x01f7c), CE(0x00301, 0x003ce), CE(0x00313, 0x01f60),
            CE(0x00314, 0x01f61), CE(0x00342, 0x01ff6), CE(0x00345, 0x01ff3),
            CE(0x00300, 0x01fd2), CE(0x00301, 0x00390), CE(0x00342, 0x01fd7),
            CE(0x00300, 0x01fe2), CE(0x00301, 0x003b0), CE(0x00342, 0x01fe7),
            CE(0x00345, 0x01ff4), CE(0x00301, 0x003d3), CE(0x00308, 0x003d4),
            CE(0x00308, 0x00407), CE(0x00306, 0x004d0), CE(0x00308, 0x004d2),
            CE(0x00301, 0x00403), CE(0x00300, 0x00400), CE(0x00306, 0x004d6),
            CE(0x00308, 0x00401), CE(0x00306, 0x004c1), CE(0x00308, 0x004dc),
            CE(0x00308, 0x004de), CE(0x00300, 0x0040d), CE(0x00304, 0x004e2),
            CE(0x00306, 0x00419), CE(0x00308, 0x004e4), CE(0x00301, 0x0040c),
            CE(0x00308, 0x004e6), CE(0x00304, 0x004ee), CE(0x00306, 0x0040e),
            CE(0x00308, 0x004f0), CE(0x0030b, 0x004f2), CE(0x00308, 0x004f4),
            CE(0x00308, 0x004f8), CE(0x00308, 0x004ec), CE(0x00306, 0x004d1),
            CE(0x00308, 0x004d3), CE(0x00301, 0x00453), CE(0x00300, 0x00450),
            CE(0x00306, 0x004d7), CE(0x00308, 0x00451), CE(0x00306, 0x004c2),
            CE(0x00308, 0x004dd), CE(0x00308, 0x004df), CE(0x00300, 0x0045d),
            CE(0x00304, 0x004e3), CE(0x00306, 0x00439), CE(0x00308, 0x004e5),
            CE(0x00301, 0x0045c), CE(0x00308, 0x004e7), CE(0x00304, 0x004ef),
            CE(0x00306, 0x0045e), CE(0x00308, 0x004f1), CE(0x0030b, 0x004f3),
            CE(0x00308, 0x004f5), CE(0x00308, 0x004f9), CE(0x00308, 0x004ed),
            CE(0x00308, 0x00457), CE(0x0030f, 0x00476), CE(0x0030f, 0x00477),
            CE(0x00308, 0x004da), CE(0x00308, 0x004db), CE(0x00308, 0x004ea),
            CE(0x00308, 0x004eb), CE(0x00653, 0x00622), CE(0x00654, 0x00623),
            CE(0x00655, 0x00625), CE(0x00654, 0x00624), CE(0x00654, 0x00626),
            CE(0x00654, 0x006c2), CE(0x00654, 0x006d3), CE(0x00654, 0x006c0),
            CE(0x0093c, 0x00929), CE(0x0093c, 0x00931), CE(0x0093c, 0x00934),
            CE(0x009be, 0x009cb), CE(0x009d7, 0x009cc), CE(0x00b3e, 0x00b4b),
            CE(0x00b56, 0x00b48), CE(0x00b57, 0x00b4c), CE(0x00bd7, 0x00b94),
            CE(0x00bbe, 0x00bca), CE(0x00bd7, 0x00bcc), CE(0x00bbe, 0x00bcb),
            CE(0x00c56, 0x00c48), CE(0x00cd5, 0x00cc0), CE(0x00cc2, 0x00cca),
            CE(0x00cd5, 0x00cc7), CE(0x00cd6, 0x00cc8), CE(0x00cd5, 0x00ccb),
            CE(0x00d3e, 0x00d4a), CE(0x00d57, 0x00d4c), CE(0x00d3e, 0x00d4b),
            CE(0x00dca, 0x00dda), CE(0x00dcf, 0x00ddc), CE(0x00ddf, 0x00dde),
            CE(0x00dca, 0x00ddd), CE(0x0102e, 0x01026), CE(0x01b35, 0x01b06),
            CE(0x01b35, 0x01b08), CE(0x01b35, 0x01b0a), CE(0x01b35, 0x01b0c),
            CE(0x01b35, 0x01b0e), CE(0x01b35, 0x01b12), CE(0x01b35, 0x01b3b),
            CE(0x01b35, 0x01b3d), CE(0x01b35, 0x01b40), CE(0x01b35, 0x01b41),
            CE(0x01b35, 0x01b43), CE(0x00304, 0x01e38), CE(0x00304, 0x01e39),
            CE(0x00304, 0x01e5c), CE(0x00304, 0x01e5d), CE(0x00307, 0x01e68),
            CE(0x00307, 0x01e69), CE(0x00302, 0x01eac), CE(0x00306, 0x01eb6),
            CE(0x00302, 0x01ead), CE(0x00306, 0x01eb7), CE(0x00302, 0x01ec6),
            CE(0x00302, 0x01ec7), CE(0x00302, 0x01ed8), CE(0x00302, 0x01ed9),
            CE(0x00300, 0x01f02), CE(0x00301, 0x01f04), CE(0x00342, 0x01f06),
            CE(0x00345, 0x01f80), CE(0x00300, 0x01f03), CE(0x00301, 0x01f05),
            CE(0x00342, 0x01f07), CE(0x00345, 0x01f81), CE(0x00345, 0x01f82),
            CE(0x00345, 0x01f83), CE(0x00345, 0x01f84), CE(0x00345, 0x01f85),
            CE(0x00345, 0x01f86), CE(0x00345, 0x01f87), CE(0x00300, 0x01f0a),
            CE(0x00301, 0x01f0c), CE(0x00342, 0x01f0e), CE(0x00345, 0x01f88),
            CE(0x00300, 0x01f0b), CE(0x00301, 0x01f0d), CE(0x00342, 0x01f0f),
            CE(0x00345, 0x01f89), CE(0x00345, 0x01f8a), CE(0x00345, 0x01f8b),
            CE(0x00345, 0x01f8c), CE(0x00345, 0x01f8d), CE(0x00345, 0x01f8e),
            CE(0x00345, 0x01f8f), CE(0x00300, 0x01f12), CE(0x00301, 0x01f14),
            CE(0x00300, 0x01f13), CE(0x00301, 0x01f15), CE(0x00300, 0x01f1a),
            CE(0x00301, 0x01f1c), CE(0x00300, 0x01f1b), CE(0x00301, 0x01f1d),
            CE(0x00300, 0x01f22), CE(0x00301, 0x01f24), CE(0x00342, 0x01f26),
            CE(0x00345, 0x01f90), CE(0x00300, 0x01f23), CE(0x00301, 0x01f25),
            CE(0x00342, 0x01f27), CE(0x00345, 0x01f91), CE(0x00345, 0x01f92),
            CE(0x00345, 0x01f93), CE(0x00345, 0x01f94), CE(0x00345, 0x01f95),
            CE(0x00345, 0x01f96), CE(0x00345, 0x01f97), CE(0x00300, 0x01f2a),
            CE(0x00301, 0x01f2c), CE(0x00342, 0x01f2e), CE(0x00345, 0x01f98),
            CE(0x00300, 0x01f2b), CE(0x00301, 0x01f2d), CE(0x00342, 0x01f2f),
            CE(0x00345, 0x01f99), CE(0x00345, 0x01f9a), CE(0x00345, 0x01f9b),
            CE(0x00345, 0x01f9c), CE(0x00345, 0x01f9d), CE(0x00345, 0x01f9e),
            CE(0x00345, 0x01f9f), CE(0x00300, 0x01f32), CE(0x00301, 0x01f34),
            CE(0x00342, 0x01f36), CE(0x00300, 0x01f33), CE(0x00301, 0x01f35),
            CE(0x00342, 0x01f37), CE(0x00300, 0x01f3a), CE(0x00301, 0x01f3c),
            CE(0x00342, 0x01f3e), CE(0x00300, 0x01f3b), CE(0x00301, 0x01f3d),
            CE(0x00342, 0x01f3f), CE(0x00300, 0x01f42), CE(0x00301, 0x01f44),
            CE(0x00300, 0x01f43), CE(0x00301, 0x01f45), CE(0x00300, 0x01f4a),
            CE(0x00301, 0x01f4c), CE(0x00300, 0x01f4b), CE(0x00301, 0x01f4d),
            CE(0x00300, 0x01f52), CE(0x00301, 0x01f54), CE(0x00342, 0x01f56),
            CE(0x00300, 0x01f53), CE(0x00301, 0x01f55), CE(0x00342, 0x01f57),
            CE(0x00300, 0x01f5b), CE(0x00301, 0x01f5d), CE(0x00342, 0x01f5f),
            CE(0x00300, 0x01f62), CE(0x00301, 0x01f64), CE(0x00342, 0x01f66),
            CE(0x00345, 0x01fa0), CE(0x00300, 0x01f63), CE(0x00301, 0x01f65),
            CE(0x00342, 0x01f67), CE(0x00345, 0x01fa1), CE(0x00345, 0x01fa2),
            CE(0x00345, 0x01fa3), CE(0x00345, 0x01fa4), CE(0x00345, 0x01fa5),
            CE(0x00345, 0x01fa6), CE(0x00345, 0x01fa7), CE(0x00300, 0x01f6a),
            CE(0x00301, 0x01f6c), CE(0x00342, 0x01f6e), CE(0x00345, 0x01fa8),
            CE(0x00300, 0x01f6b), CE(0x00301, 0x01f6d), CE(0x00342, 0x01f6f),
            CE(0x00345, 0x01fa9), CE(0x00345, 0x01faa), CE(0x00345, 0x01fab),
            CE(0x00345, 0x01fac), CE(0x00345, 0x01fad), CE(0x00345, 0x01fae),
            CE(0x00345, 0x01faf), CE(0x00345, 0x01fb2), CE(0x00345, 0x01fc2),
            CE(0x00345, 0x01ff2), CE(0x00345, 0x01fb7), CE(0x00300, 0x01fcd),
            CE(0x00301, 0x01fce), CE(0x00342, 0x01fcf), CE(0x00345, 0x01fc7),
            CE(0x00345, 0x01ff7), CE(0x00300, 0x01fdd), CE(0x00301, 0x01fde),
            CE(0x00342, 0x01fdf), CE(0x00338, 0x0219a), CE(0x00338, 0x0219b),
            CE(0x00338, 0x021ae), CE(0x00338, 0x021cd), CE(0x00338, 0x021cf),
            CE(0x00338, 0x021ce), CE(0x00338, 0x02204), CE(0x00338, 0x02209),
            CE(0x00338, 0x0220c), CE(0x00338, 0x02224), CE(0x00338, 0x02226),
            CE(0x00338, 0x02241), CE(0x00338, 0x02244), CE(0x00338, 0x02247),
            CE(0x00338, 0x02249), CE(0x00338, 0x0226d), CE(0x00338, 0x02262),
            CE(0x00338, 0x02270), CE(0x00338, 0x02271), CE(0x00338, 0x02274),
            CE(0x00338, 0x02275), CE(0x00338, 0x02278), CE(0x00338, 0x02279),
            CE(0x00338, 0x02280), CE(0x00338, 0x02281), CE(0x00338, 0x022e0),
            CE(0x00338, 0x022e1), CE(0x00338, 0x02284), CE(0x00338, 0x02285),
            CE(0x00338, 0x02288), CE(0x00338, 0x02289), CE(0x00338, 0x022e2),
            CE(0x00338, 0x022e3), CE(0x00338, 0x022ac), CE(0x00338, 0x022ad),
            CE(0x00338, 0x022ae), CE(0x00338, 0x022af), CE(0x00338, 0x022ea),
            CE(0x00338, 0x022eb), CE(0x00338, 0x022ec), CE(0x00338, 0x022ed),
            CE(0x03099, 0x03094), CE(0x03099, 0x0304c), CE(0x03099, 0x0304e),
            CE(0x03099, 0x03050), CE(0x03099, 0x03052), CE(0x03099, 0x03054),
            CE(0x03099, 0x03056), CE(0x03099, 0x03058), CE(0x03099, 0x0305a),
            CE(0x03099, 0x0305c), CE(0x03099, 0x0305e), CE(0x03099, 0x03060),
            CE(0x03099, 0x03062), CE(0x03099, 0x03065), CE(0x03099, 0x03067),
            CE(0x03099, 0x03069), CE(0x03099, 0x03070), CE(0x0309a, 0x03071),
            CE(0x03099, 0x03073), CE(0x0309a, 0x03074), CE(0x03099, 0x03076),
            CE(0x0309a, 0x03077), CE(0x03099, 0x03079), CE(0x0309a, 0x0307a),
            CE(0x03099, 0x0307c), CE(0x0309a, 0x0307d), CE(0x03099, 0x0309e),
            CE(0x03099, 0x030f4), CE(0x03099, 0x030ac), CE(0x03099, 0x030ae),
            CE(0x03099, 0x030b0), CE(0x03099, 0x030b2), CE(0x03099, 0x030b4),
            CE(0x03099, 0x030b6), CE(0x03099, 0x030b8), CE(0x03099, 0x030ba),
            CE(0x03099, 0x030bc), CE(0x03099, 0x030be), CE(0x03099, 0x030c0),
            CE(0x03099, 0x030c2), CE(0x03099, 0x030c5), CE(0x03099, 0x030c7),
            CE(0x03099, 0x030c9), CE(0x03099, 0x030d0), CE(0x0309a, 0x030d1),
            CE(0x03099, 0x030d3), CE(0x0309a, 0x030d4), CE(0x03099, 0x030d6),
            CE(0x0309a, 0x030d7), CE(0x03099, 0x030d9), CE(0x0309a, 0x030da),
            CE(0x03099, 0x030dc), CE(0x0309a, 0x030dd), CE(0x03099, 0x030f7),
            CE(0x03099, 0x030f8), CE(0x03099, 0x030f9), CE(0x03099, 0x030fa),
            CE(0x03099, 0x030fe), CE(0x110ba, 0x1109a), CE(0x110ba, 0x1109c),
            CE(0x110ba, 0x110ab), CE(0x11127, 0x1112e), CE(0x11127, 0x1112f),
        ];
        return t;
    }

}
